@@ -36,9 +36,38 @@ class XmlCoverageReporter(BaseViolationReporter):
         # Keys are source file paths, values are output of `violations()`
         self._info_cache = defaultdict(list)
 
+        # Create a list to cache xml classes list results
+        # Values are output of `self._get_xml_classes()`
+        self._xml_cache = [{} for i in range(len(xml_roots))]
+
         self._src_roots = src_roots or [""]
 
-    def _get_classes(self, xml_document, src_path):
+    def _get_xml_classes(self, xml_document):
+        """
+        Return a dict of classes in `xml_document`.
+        Keys are `filename`, values are list of `class`
+
+        If `class` is not present in `xml_document`,
+        return empty defaultdict(list)
+        """
+        # cobertura sometimes provides the sources for the measurements
+        # within it. If we have that we outta use it
+        sources = xml_document.findall("sources/source")
+        sources = [source.text for source in sources if source.text]
+        classes = xml_document.findall(".//class") or []
+
+        res = defaultdict(list)
+        for clazz in classes:
+            f = clazz.get("filename")
+            if not f:
+                continue
+            res[util.to_unix_path(f)].append(clazz)
+            for source in sources:
+                abs_f = util.to_unix_path(os.path.join(source.strip(), f))
+                res[abs_f].append(clazz)
+        return res
+
+    def _get_classes(self, index, xml_document, src_path):
         """
         Given a path and parsed xml_document provides class nodes
         with the relevant lines
@@ -63,38 +92,16 @@ class XmlCoverageReporter(BaseViolationReporter):
         # search for `/home/user/work/diff-cover/other_package/some_file.py`
         src_abs_path = util.to_unix_path(GitPathTool.absolute_path(src_path))
 
-        # cobertura sometimes provides the sources for the measurements
-        # within it. If we have that we outta use it
-        sources = xml_document.findall("sources/source")
-        sources = [source.text for source in sources if source.text]
-        classes = xml_document.findall(".//class") or []
+        # Create a cache for `classes` in `xml_document` if cache exists
+        if not self._xml_cache[index]:
+            self._xml_cache[index] = self._get_xml_classes(xml_document)
 
-        return (
-            [
-                clazz
-                for clazz in classes
-                if src_abs_path
-                in [
-                    util.to_unix_path(
-                        os.path.join(source.strip(), clazz.get("filename"))
-                    )
-                    for source in sources
-                ]
-            ]
-            or [
-                clazz
-                for clazz in classes
-                if util.to_unix_path(clazz.get("filename")) == src_abs_path
-            ]
-            or [
-                clazz
-                for clazz in classes
-                if util.to_unix_path(clazz.get("filename")) == src_rel_path
-            ]
+        return self._xml_cache[index].get(src_abs_path) or self._xml_cache[index].get(
+            src_rel_path
         )
 
-    def get_src_path_line_nodes_cobertura(self, xml_document, src_path):
-        classes = self._get_classes(xml_document, src_path)
+    def get_src_path_line_nodes_cobertura(self, index, xml_document, src_path):
+        classes = self._get_classes(index, xml_document, src_path)
 
         if not classes:
             return None
@@ -184,7 +191,7 @@ class XmlCoverageReporter(BaseViolationReporter):
             measured = set()
 
             # Loop through the files that contain the xml roots
-            for xml_document in self._xml_roots:
+            for i, xml_document in enumerate(self._xml_roots):
                 if xml_document.findall(".[@clover]"):
                     # see etc/schema/clover.xsd at  https://bitbucket.org/atlassian/clover/src
                     line_nodes = self.get_src_path_line_nodes_clover(
@@ -202,7 +209,7 @@ class XmlCoverageReporter(BaseViolationReporter):
                 else:
                     # https://github.com/cobertura/web/blob/master/htdocs/xml/coverage-04.dtd
                     line_nodes = self.get_src_path_line_nodes_cobertura(
-                        xml_document, src_path
+                        i, xml_document, src_path
                     )
                     _number = "number"
                     _hits = "hits"
@@ -255,6 +262,169 @@ class XmlCoverageReporter(BaseViolationReporter):
         return self._info_cache[src_path][1]
 
 
+class LcovCoverageReporter(BaseViolationReporter):
+    """
+    Query information from a Cobertura|Clover|JaCoCo XML coverage report.
+    """
+
+    def __init__(self, lcov_roots, src_roots=None):
+        """
+        Load the lcov.info coverage report represented
+        """
+        super().__init__("LCOV")
+        self._lcov_roots = lcov_roots
+        self._lcov_report = defaultdict(list)
+
+        # Create a dict to cache violations dict results
+        # Keys are source file paths, values are output of `violations()`
+        self._info_cache = defaultdict(list)
+
+        self._src_roots = src_roots or [""]
+
+    @staticmethod
+    def parse(lcov_file):
+        """
+        Parse a single LCov coverage report
+        File format: https://ltp.sourceforge.net/coverage/lcov/geninfo.1.php
+        """
+        lcov_report = defaultdict(dict)
+        lcov = open(lcov_file, "r")
+        while True:
+            line = lcov.readline()
+            if not line:
+                break
+            directive, _, content = line.strip().partition(":")
+            # we're only interested in file name and line coverage
+            if directive == "SF":
+                # SF:<absolute path to the source file>
+                source_file = content
+                continue
+            elif directive == "DA":
+                # DA:<line number>,<execution count>[,<checksum>]
+                args = content.split(",")
+                if len(args) < 2 or len(args) > 3:
+                    raise ValueError(f"Unknown syntax in lcov report: {line}")
+                line_no = int(args[0])
+                num_executions = int(args[1])
+                if source_file is None:
+                    raise ValueError(
+                        f"No source file specified for line coverage: {line}"
+                    )
+                if line_no not in lcov_report[source_file]:
+                    lcov_report[source_file][line_no] = 0
+                lcov_report[source_file][line_no] += num_executions
+            elif directive in [
+                "TN",
+                "FNF",
+                "FNH",
+                "FN",
+                "FNDA",
+                "LH",
+                "LF",
+                "BRF",
+                "BRH",
+                "BRDA",
+            ]:
+                # these are valid lines, but not we don't need them
+                continue
+            elif directive == "end_of_record":
+                source_file = None
+            else:
+                raise ValueError(f"Unknown syntax in lcov report: {line}")
+
+        lcov.close()
+        return lcov_report
+
+    def _cache_file(self, src_path):
+        """
+        Load the data from `self._lcov_roots`
+        for `src_path`, if it hasn't been already.
+        """
+        # If we have not yet loaded this source file
+        if src_path not in self._info_cache:
+            # We only want to keep violations that show up in each xml source.
+            # Thus, each time, we take the intersection.  However, to do this
+            # we must treat the first time as a special case and just add all
+            # the violations from the first xml report.
+            violations = None
+
+            # A line is measured if it is measured in any of the reports, so
+            # we take set union each time and can just start with the empty set
+            measured = set()
+
+            # Remove git_root from src_path for searching the correct filename
+            # If cwd is `/home/user/work/diff-cover/diff_cover`
+            # and src_path is `diff_cover/violations_reporter.py`
+            # search for `violations_reporter.py`
+            src_rel_path = util.to_unix_path(GitPathTool.relative_path(src_path))
+
+            # If cwd is `/home/user/work/diff-cover/diff_cover`
+            # and src_path is `other_package/some_file.py`
+            # search for `/home/user/work/diff-cover/other_package/some_file.py`
+            src_abs_path = util.to_unix_path(GitPathTool.absolute_path(src_path))
+
+            # Loop through the files that contain the xml roots
+            for lcov_document in self._lcov_roots:
+                src_search_path = src_abs_path
+                if src_search_path not in lcov_document:
+                    src_search_path = src_rel_path
+
+                # First case, need to define violations initially
+                if violations is None:
+                    violations = {
+                        Violation(int(line_no), None)
+                        for line_no, num_executions in lcov_document[
+                            src_search_path
+                        ].items()
+                        if int(num_executions) == 0
+                    }
+
+                # If we already have a violations set,
+                # take the intersection of the new
+                # violations set and its old self
+                else:
+                    violations = violations & {
+                        Violation(int(line_no), None)
+                        for line_no, num_executions in lcov_document[
+                            src_search_path
+                        ].items()
+                        if int(num_executions) == 0
+                    }
+
+                # Measured is the union of itself and the new measured
+                # measured = measured | {int(line.get(_number)) for line in line_nodes}
+                measured = measured | {
+                    int(line_no)
+                    for line_no, num_executions in lcov_document[
+                        src_search_path
+                    ].items()
+                }
+
+            # If we don't have any information about the source file,
+            # don't report any violations
+            if violations is None:
+                violations = set()
+
+            self._info_cache[src_path] = (violations, measured)
+
+    def violations(self, src_path):
+        """
+        See base class comments.
+        """
+
+        self._cache_file(src_path)
+
+        # Yield all lines not covered
+        return self._info_cache[src_path][0]
+
+    def measured_lines(self, src_path):
+        """
+        See base class docstring.
+        """
+        self._cache_file(src_path)
+        return self._info_cache[src_path][1]
+
+
 pycodestyle_driver = RegexBasedDriver(
     name="pycodestyle",
     supported_extensions=["py"],
@@ -273,7 +443,7 @@ pyflakes_driver = RegexBasedDriver(
     # Match lines of the form:
     # path/to/file.py:328: undefined name '_thing'
     # path/to/file.py:418: 'random' imported but unused
-    expression=r"^([^:]+):(\d+):\d* (.*)$",
+    expression=r"^([^:]+):(\d+):\d*:? (.*)$",
     command_to_check_install=["pyflakes", "--version"],
     # pyflakes exit code is 1 if there are violations
     # https://github.com/PyCQA/pyflakes/blob/master/pyflakes/api.py#L211
@@ -402,7 +572,7 @@ class PylintDriver(QualityDriver):
         # Match lines of the form:
         # path/to/file.py:123: [C0111] Missing docstring
         # path/to/file.py:456: [C0111, Foo.bar] Missing docstring
-        self.multi_line_violation_regex = re.compile(r"==(\w|.+):(.*)")
+        self.multi_line_violation_regex = re.compile(r"==((?:\w|\.)+?):\[?(\d+)")
         self.dupe_code_violation_regex = re.compile(r"Similar lines in (\d+) files")
 
     def _process_dupe_code_violation(self, lines, current_line, message):
@@ -438,7 +608,6 @@ class PylintDriver(QualityDriver):
                 # Ignore any line that isn't matched
                 # (for example, snippets from the source code)
                 if match is not None:
-
                     (
                         pylint_src_path,
                         line_number,
@@ -521,7 +690,6 @@ class CppcheckDriver(QualityDriver):
                 # Ignore any line that isn't matched
                 # (for example, snippets from the source code)
                 if match is not None:
-
                     (cppcheck_src_path, line_number, message) = match.groups()
 
                     violation = Violation(int(line_number), message)
